# coding: utf-8
import json
from datetime import datetime
from uuid import uuid4
from ru.curs.celesta.showcase.utils import XMLJSONConverter
try:
    from ru.curs.showcase.core.jython import JythonDTO
    from ru.curs.showcase.core.selector import ResultSelectorData
    from ru.beta2.extra.gwt.ui.selector.api import DataRecord
except:
    from ru.curs.celesta.showcase import JythonDTO
from nci import SUPERVISED_STUDENT, generalSettings
from dirusing.commonfunctions import relatedTableCursorImport
from edu.constants import *    
from nci._nci_orm import vw_status_model_statusCursor,\
    person_edu_organizationCursor    
from edu._edu_orm import course_cycleCursor, course_cycle_qti_packageCursor
from medstaff._medstaff_orm import student_certificateCursor,\
    vw_certificateCursor, student_programCursor, student_cycleCursor,\
    student_eor_tasksCursor    


def cardData(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    u'''Данные для карточки.'''
    record_id = json.loads(session)['sessioncontext']['related']['gridContext']['currentRecordId']
    student_certificate = student_certificateCursor(context)
    student_certificate.get(record_id)
    current_status = student_certificate.status
    # Структура данных
    xformsdata = {"schema": {
                    "@xmlns": "",
                    "data": {
                        "record_id": record_id,
                        "oldstatus": current_status,
                        "newstatus": current_status,
                        "comment": ""}}}

    # действия в зависимости от статуса
    vw_status_model_status = vw_status_model_statusCursor(context)
    vw_status_model_status.setRange('model_id', 4)
    vw_status_model_status.setRange('status_id1', student_certificate.status)
    xformsdata['schema']['data']['actions'] = []
    xformsdata['schema']['data']['actions'].append({'@label': u'Оставить в текущем статусе', '@value': current_status})
    for vw_status_model_status in vw_status_model_status.iterate():
        xformsdata['schema']['data']['actions'].append({'@label': vw_status_model_status.transition_text,
                                                     '@value': int(vw_status_model_status.status_id2)})
    # Первоначальная xformssettings
    xformssettings = {"properties": {
                        "event": [
                            {"@name": "single_click",
                             "@linkId": "1",
                             "action": {
                                "@keep_user_settings": "true",
                                '#sorted': [
                                    {"main_context": "current"},
                                    {"datapanel": {
                                        "@type": "current",
                                        "@tab": "current",
                                        "element": {
                                            "@id":'certificationMethodGrid',
                                            "add_context": ""}}}]}},
                             {"@name": "single_click",
                             "@linkId": "2",
                             "action": {
                                "@keep_user_settings": "true",
                                '#sorted': [
                                    {"main_context": "current"},
                                    {"datapanel": {
                                        "@type": "current",
                                        "@tab": "current",
                                        "element": { 
                                            "@id":elementId,
                                            "add_context": "hide"}}}]}}]}}

    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(xformsdata)), XMLJSONConverter.jsonToXml(json.dumps(xformssettings)))


def cardSave(context, main=None, add=None, filterinfo=None, session=None, elementId=None, xformsdata=None):
    u'''Сохранение карточки. '''
    data = json.loads(xformsdata)['schema']['data']
    student_certificate = student_certificateCursor(context)
    certificateId = data['record_id']
    student_certificate.get(certificateId)
    if data['newstatus'] != data['oldstatus']:
        student_certificate.status = data['newstatus']
        student_certificate.update()
        student_certificate.close()
        # Если слушатель допущен к обучению, то привязываем его к соответствующей организации
        if int(data['newstatus']) == 20:           
            certificateCur = vw_certificateCursor(context)
            certificateCur.setRange('uid', certificateId)
            certificateCur.first()
            edu_organization_id = certificateCur.edu_org_id
            person_id = certificateCur.person_id
            certificateCur.close()
            # Привязываем слушателя к образовательной организации и формируем ему программу обучения
            if edu_organization_id and person_id:
                # Привязываем слушателя к образовательной организации
                person_edu_organization = person_edu_organizationCursor(context)
                person_edu_organization.setRange('edu_organization_id', edu_organization_id)
                person_edu_organization.setRange('person_id', person_id)
                if not person_edu_organization.tryFirst():
                    person_edu_organization.uid = uuid4()
                    person_edu_organization.person_id = person_id
                    person_edu_organization.edu_organization_id = edu_organization_id
                    person_edu_organization.role = SUPERVISED_STUDENT
                    person_edu_organization.insert()
                
                person_edu_organization.close()
                
                # Формируем программу
                addStudentProgram(context, certificateId)
                

def addStudentProgram(context, certificate):
    student_certificate = student_certificateCursor(context)
    if student_certificate.tryGet(certificate):
        person_id = student_certificate.person_id
        course_id = student_certificate.course_id
        
        attempts_count = generalSettings.getGlobalSettings(context, 'TEST_ATTEMPTS_COUNT')
        if not attempts_count:
            context.error(u'Ошибка при получении значения глобального параметра "Количество попыток прохождения теста"')
        else:
            attempts_count = int(attempts_count)
        
        student_program = student_programCursor(context)
        student_program.setRange('person_id', person_id)
        student_program.setRange('course_id', course_id)
        
        if not student_program.tryGet(person_id, course_id, certificate):
            
            currentTable = {}
            for key in EOR_TYPE.keys():
                currentTable[key] = relatedTableCursorImport(EOR_TYPE[key][1], EOR_TYPE[key][2])(context)
            
            
            student_program.person_id = person_id
            student_program.course_id = course_id
            student_program.course_open = True
            student_program.start_date = datetime.now()
            student_program.student_certificate = certificate
            student_program.insert()
            student_cycle = student_cycleCursor(context)
            course_cycle = course_cycleCursor(context)
            
            cycle_qti = course_cycle_qti_packageCursor(context)
            
            student_qty = student_eor_tasksCursor(context)            
            course_cycle.setRange('course_id', course_id)
            course_cycle.orderBy('cycle_order')
            cycle_open = True
            for cycle in course_cycle.iterate():
                cycle_id = cycle.uid
                student_cycle.person_id = person_id
                student_cycle.cycle_id = cycle_id
                student_cycle.course_id = course_id
                student_cycle.student_certificate = certificate
                student_cycle.cycle_order = cycle.cycle_order
                # Устанавливаем первой по порядку теме признак доступности
                student_cycle.cycle_open = cycle_open
                if cycle_open:
                    cycle_open = False
                student_cycle.insert()                
                # Генерация вариантов для тестирования
                cycle_qti.setRange('course_cycle_id', cycle_id)
                for qti_item in cycle_qti.iterate():
                    student_qty.clear()
                    student_qty.student_certificate = certificate
                    student_qty.cycle_id = cycle_id
                    student_qty.package_id = qti_item.qti_package_id
                    
                    eor_type = qti_item.eor_type
                    student_qty.eor_type = eor_type
                    
                    name = u''
                    if currentTable[eor_type].tryGet(qti_item.qti_package_id):
                        name = getattr(currentTable[eor_type], EOR_TYPE[eor_type][3])
                    
                    student_qty.package_name = name
                    student_qty.control_type = qti_item.control_type
                    student_qty.qti_order = qti_item.qti_order
                    
                    if eor_type == TEST:
                        student_qty.attempts_count = attempts_count 
                    else:
                        student_qty.attempts_count = 0
                        
                    student_qty.insert()                
                
            student_qty.close()
            cycle_qti.close()
            student_cycle.close()
            course_cycle.close()
            
        student_program.close()
    student_certificate.close()             
